/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.apilib.vorgang.grpc;

import java.util.Collections;
import java.util.HashSet;
import java.util.Optional;
import java.util.Set;

import org.mapstruct.CollectionMappingStrategy;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.NullValueCheckStrategy;
import org.mapstruct.ReportingPolicy;

import de.ozgcloud.apilib.vorgang.OzgCloudVorgangQuery;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangQuery.FormIdentification;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangQuery.Query;
import de.ozgcloud.vorgang.common.GrpcQueryOperator;
import de.ozgcloud.vorgang.vorgang.GrpcLogicalOperator;
import de.ozgcloud.vorgang.vorgang.GrpcQuery;
import de.ozgcloud.vorgang.vorgang.GrpcVorgangQueryExpression;
import de.ozgcloud.vorgang.vorgang.GrpcVorgangQueryExpression.Builder;

@Mapper(collectionMappingStrategy = CollectionMappingStrategy.ADDER_PREFERRED, nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS, unmappedTargetPolicy = ReportingPolicy.WARN)
public interface OzgCloudVorgangQueryMapper {

	String FORM_ENGINE_NAME_PATH = "Vorgang.eingangs.header.formEngineName";
	String FORM_ID_PATH = "Vorgang.eingangs.header.formId";
	String STATUS_PATH = "Vorgang.status";
	String ORGANISATIONS_EINHEIT_ID_PATH = "Vorgang.eingangs.zustaendigeStelle.organisationseinheitenId";
	String AKTENZEICHEN_PATH = "Vorgang.aktenzeichen";

	default GrpcQuery toGrpc(OzgCloudVorgangQuery ozgCloudVorgangQuery) {
		return hasPropertyExpressions(ozgCloudVorgangQuery)
				? mapAllToQuery(ozgCloudVorgangQuery)
				: ozgCloudVorgangQuery.getQuery().map(this::toQuery).orElseGet(GrpcQuery::getDefaultInstance);
	}

	default boolean hasPropertyExpressions(OzgCloudVorgangQuery query) {
		return query.getAktenzeichen().isPresent() || query.getForm().isPresent() || query.getStatus().isPresent()
				|| query.getOrganisationsEinheitId().isPresent();
	}

	default GrpcQuery mapAllToQuery(OzgCloudVorgangQuery query) {
		var queryBuilder = GrpcQuery.newBuilder().setLogicalOperator(GrpcLogicalOperator.AND);
		query.getAktenzeichen().map(this::toAktenzeichenExpression).ifPresent(queryBuilder::addExpressions);
		query.getForm().map(this::toFormExpressions).ifPresent(queryBuilder::addAllExpressions);
		query.getStatus().map(this::toStatusExpression).ifPresent(queryBuilder::addExpressions);
		query.getOrganisationsEinheitId().map(this::toOrganisationsEinheitIdExpression).ifPresent(queryBuilder::addExpressions);
		query.getQuery().map(this::toQuery).ifPresent(queryBuilder::setNestedQuery);
		return queryBuilder.build();
	}

	default GrpcVorgangQueryExpression toAktenzeichenExpression(String aktenzeichen) {
		return getEqualExpressionBuilder()
				.setPath(AKTENZEICHEN_PATH)
				.setOperandStringValue(aktenzeichen)
				.build();
	}

	default GrpcVorgangQueryExpression toOrganisationsEinheitIdExpression(String organisationsEinheitId) {
		return getEqualExpressionBuilder()
				.setPath(ORGANISATIONS_EINHEIT_ID_PATH)
				.setOperandStringValue(organisationsEinheitId)
				.build();
	}

	default GrpcVorgangQueryExpression toStatusExpression(String status) {
		return getEqualExpressionBuilder()
				.setPath(STATUS_PATH)
				.setOperandStringValue(status)
				.build();
	}

	default Set<GrpcVorgangQueryExpression> toFormExpressions(FormIdentification formIdentification) {
		var expressions = HashSet.<GrpcVorgangQueryExpression>newHashSet(2);
		Optional.ofNullable(formIdentification.getFormEngineName()).map(this::toFormEngineNameExpression).ifPresent(expressions::add);
		Optional.ofNullable(formIdentification.getFormId()).map(this::toFormIdExpression).ifPresent(expressions::add);
		return Collections.unmodifiableSet(expressions);
	}

	default GrpcVorgangQueryExpression toFormEngineNameExpression(String formEngineName) {
		return getEqualExpressionBuilder()
				.setPath(FORM_ENGINE_NAME_PATH)
				.setOperandStringValue(formEngineName)
				.build();
	}

	default GrpcVorgangQueryExpression toFormIdExpression(String formId) {
		return getEqualExpressionBuilder()
				.setPath(FORM_ID_PATH)
				.setOperandStringValue(formId)
				.build();
	}

	private Builder getEqualExpressionBuilder() {
		return GrpcVorgangQueryExpression.newBuilder()
				.setOperator(GrpcQueryOperator.EQUAL);
	}

	@Mapping(target = "mergeFrom", ignore = true)
	@Mapping(target = "clearField", ignore = true)
	@Mapping(target = "clearOneof", ignore = true)
	@Mapping(target = "mergeNestedQuery", ignore = true)
	@Mapping(target = "mergeUnknownFields", ignore = true)
	@Mapping(target = "removeExpressions", ignore = true)
	@Mapping(target = "logicalOperatorValue", ignore = true)
	@Mapping(target = "unknownFields", ignore = true)
	@Mapping(target = "allFields", ignore = true)
	@Mapping(target = "expressionsBuilderList", ignore = true)
	@Mapping(target = "expressionsOrBuilderList", ignore = true)
	@Mapping(target = "nestedQueryOrBuilder", ignore = true)
	@Mapping(target = "defaultInstanceForType", ignore = true)

	@Mapping(target = "expressionsList", source = "expressions")
	GrpcQuery toQuery(Query query);
}
