/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.apilib.common.command.grpc;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.apilib.common.command.GrpcCreateCommandTestFactory;
import de.ozgcloud.apilib.common.command.OzgCloudCommand;
import de.ozgcloud.apilib.common.command.OzgCloudCreateSubCommandsRequestTestFactory;
import de.ozgcloud.vorgang.grpc.command.GrpcCreateCommand;

class CommandMapperTest {

	@Spy
	@InjectMocks
	private CommandMapper mapper = Mappers.getMapper(CommandMapper.class);

	@Nested
	class TestBuildCreateCommandRequest {

		@Test
		void shouldBuildRequest() {
			var request = mapper.buildCreateCommandRequest(OzgCloudCommandTestFactory.create());

			assertThat(request).usingRecursiveComparison().isEqualTo(GrpcCreateCommandRequestTestFactory.create());
		}
	}

	@Nested
	class TestMapFromGrpc {

		@Test
		void shouldMap() {
			var result = mapper.fromGrpc(GrpcCommandTestFactory.create());

			assertThat(result).usingRecursiveComparison().ignoringFields("relationVersion").isEqualTo(OzgCloudCommandTestFactory.create());
		}
	}

	@Nested
	class TestToAddSubCommandsRequest {

		@Captor
		private ArgumentCaptor<List<OzgCloudCommand>> subCommandsCaptor;

		@Test
		void shouldMapParentId() {
			var request = mapper.toAddSubCommandsRequest(OzgCloudCreateSubCommandsRequestTestFactory.create());

			assertThat(request.getParentId()).isEqualTo(OzgCloudCreateSubCommandsRequestTestFactory.PARENT_ID);
		}

		@Test
		void shouldMapExecutionMode() {
			var request = mapper.toAddSubCommandsRequest(OzgCloudCreateSubCommandsRequestTestFactory.create());

			assertThat(request.getExecutionMode()).isEqualTo(OzgCloudCreateSubCommandsRequestTestFactory.EXECUTION_MODE);
		}

		@Test
		void shouldMapCompletedIfSubsCompleted() {
			var request = mapper.toAddSubCommandsRequest(OzgCloudCreateSubCommandsRequestTestFactory.create());

			assertThat(request.getCompletedIfSubsCompleted()).isEqualTo(OzgCloudCreateSubCommandsRequestTestFactory.COMPLETED_IF_SUBS_COMPLETED);
		}

		@Test
		void shouldCallMapSubCommands() {
			var subCommand = OzgCloudCommandTestFactory.create();
			var createSubCommandsRequest = OzgCloudCreateSubCommandsRequestTestFactory.createBuilder().clearSubCommands().subCommand(subCommand)
				.build();

			mapper.toAddSubCommandsRequest(createSubCommandsRequest);

			verify(mapper).toCreateCommands(subCommandsCaptor.capture());
			assertThat(subCommandsCaptor.getValue()).containsExactly(subCommand);
		}
	}

	@Nested
	class TestToCreateCommands {

		@Mock
		private GrpcCreateCommand createCommand;

		@Test
		void shouldCallToCreateCommandRequest() {
			var subCommand = OzgCloudCommandTestFactory.create();

			mapper.toCreateCommands(List.of(subCommand));

			verify(mapper).toCreateCommand(subCommand);
		}

		@Test
		void shouldReturnResult() {
			var subCommand = OzgCloudCommandTestFactory.create();
			doReturn(createCommand).when(mapper).toCreateCommand(subCommand);

			var result = mapper.toCreateCommands(List.of(subCommand));

			assertThat(result).containsExactly(createCommand);
		}
	}

	@Nested
	class TestToCreateCommand {

		@Test
		void shouldMap() {
			var command = OzgCloudCommandTestFactory.create();

			var result = mapper.toCreateCommand(command);

			assertThat(result).usingRecursiveComparison().isEqualTo(GrpcCreateCommandTestFactory.creatio());
		}

	}
}
